@@ -14,6 +14,7 @@ __license__ = 'LGPL 3+'
 
 import time
 import sys
+import types
 import importlib
 
 # we do not use this ourselves, but mock methods often want to use this
@@ -27,6 +28,7 @@ import dbus.service
 objects = {}
 
 MOCK_IFACE = 'org.freedesktop.DBus.Mock'
+OBJECT_MANAGER_IFACE = 'org.freedesktop.DBus.ObjectManager'
 
 # stubs to keep code compatible with Python 2 and 3
 if sys.version_info[0] >= 3:
@@ -56,7 +58,8 @@ class DBusMockObject(dbus.service.Object):
     that you can control the mock from any programming language.
     '''
 
-    def __init__(self, bus_name, path, interface, props, logfile=None):
+    def __init__(self, bus_name, path, interface, props, logfile=None,
+                 is_object_manager=False):
         '''Create a new DBusMockObject
 
         bus_name: A dbus.service.BusName instance where the object will be put on
@@ -69,35 +72,68 @@ class DBusMockObject(dbus.service.Object):
                  if None, logging will be written to stdout. Note that you can
                  also query the called methods over D-BUS with GetCalls() and
                  GetMethodCalls().
+        is_object_manager: If True, the GetManagedObjects method will
+                           automatically be implemented on the object, returning
+                           all objects which have this one’s path as a prefix of
+                           theirs. Note that the InterfacesAdded and
+                           InterfacesRemoved signals will not be automatically
+                           emitted.
         '''
         dbus.service.Object.__init__(self, bus_name, path)
 
         self.bus_name = bus_name
+        self.path = path
         self.interface = interface
-        # interface -> name -> value
-        self.props = {}
-        if props is None:
-            props = {}
-        self.props[interface] = props
+        self.is_object_manager = is_object_manager
 
-        # interface -> name -> (in_signature, out_signature, code, dbus_wrapper_fn)
-        self.methods = {interface: {}}
+        self._template = None
+        self._template_parameters = None
 
         if logfile:
             self.logfile = open(logfile, 'w')
         else:
             self.logfile = None
+        self.is_logfile_owner = True
         self.call_log = []
 
+        if props is None:
+            props = {}
+
+        self._reset(props)
+
     def __del__(self):
-        if self.logfile:
+        if self.logfile and self.is_logfile_owner:
             self.logfile.close()
 
+    def _set_up_object_manager(self):
+        '''Set up this mock object as a D-Bus ObjectManager.'''
+        if self.path == '/':
+            cond = 'k != \'/\''
+        else:
+            cond = 'k.startswith(\'%s/\')' % self.path
+
+        self.AddMethod(OBJECT_MANAGER_IFACE,
+                       'GetManagedObjects', '', 'a{oa{sa{sv}}}',
+                       'ret = {dbus.ObjectPath(k): objects[k].props ' +
+                       '  for k in objects.keys() if ' + cond + '}')
+
+    def _reset(self, props):
+        # interface -> name -> value
+        self.props = {self.interface: props}
+
+        # interface -> name -> (in_signature, out_signature, code, dbus_wrapper_fn)
+        self.methods = {self.interface: {}}
+
+        if self.is_object_manager:
+            self._set_up_object_manager()
+
     @dbus.service.method(dbus.PROPERTIES_IFACE,
                          in_signature='ss', out_signature='v')
     def Get(self, interface_name, property_name):
         '''Standard D-Bus API for getting a property value'''
 
+        self.log('Get %s.%s' % (interface_name, property_name))
+
         if not interface_name:
             interface_name = self.interface
         try:
@@ -112,6 +148,8 @@ class DBusMockObject(dbus.service.Object):
     def GetAll(self, interface_name, *args, **kwargs):
         '''Standard D-Bus API for getting all property values'''
 
+        self.log('GetAll ' + interface_name)
+
         if not interface_name:
             interface_name = self.interface
         try:
@@ -126,6 +164,10 @@ class DBusMockObject(dbus.service.Object):
     def Set(self, interface_name, property_name, value, *args, **kwargs):
         '''Standard D-Bus API for setting a property value'''
 
+        self.log('Set %s.%s%s' % (interface_name,
+                                  property_name,
+                                  self.format_args((value,))))
+
         try:
             iface_props = self.props[interface_name]
         except KeyError:
@@ -155,6 +197,11 @@ class DBusMockObject(dbus.service.Object):
                  methods to add to "interface"; see AddMethod() for details of
                  the tuple values
 
+        If this is a D-Bus ObjectManager instance, the InterfacesAdded signal
+        will *not* be emitted for the object automatically; it must be emitted
+        manually if desired. This is because AddInterface may be called after
+        AddObject, but before the InterfacesAdded signal should be emitted.
+
         Example:
         dbus_proxy.AddObject('/com/example/Foo/Manager',
                              'com.example.Foo.Control',
@@ -178,6 +225,7 @@ class DBusMockObject(dbus.service.Object):
                              properties)
         # make sure created objects inherit the log file stream
         obj.logfile = self.logfile
+        obj.is_logfile_owner = False
         obj.AddMethods(interface, methods)
 
         objects[path] = obj
@@ -186,15 +234,48 @@ class DBusMockObject(dbus.service.Object):
                          in_signature='s',
                          out_signature='')
     def RemoveObject(self, path):
-        '''Remove a D-Bus object from the mock'''
+        '''Remove a D-Bus object from the mock
 
+        As with AddObject, this will *not* emit the InterfacesRemoved signal if
+        it’s an ObjectManager instance.
+        '''
         try:
+            objects[path].remove_from_connection()
             del objects[path]
         except KeyError:
             raise dbus.exceptions.DBusException(
                 'org.freedesktop.DBus.Mock.NameError',
                 'object %s does not exist' % path)
 
+    @dbus.service.method(MOCK_IFACE,
+                         in_signature='', out_signature='')
+    def Reset(self):
+        '''Reset the mock object state.
+
+        Remove all mock objects from the bus and tidy up so the state is as if
+        python-dbusmock had just been restarted. If the mock object was
+        originally created with a template (from the command line, the Python
+        API or by calling AddTemplate over D-Bus), it will be
+        re-instantiated with that template.
+        '''
+        # Clear other existing objects.
+        for obj_name, obj in objects.items():
+            if obj_name != self.path:
+                obj.remove_from_connection()
+        objects.clear()
+
+        # Reinitialise our state. Carefully remove new methods from our dict.
+        for interface, methods in self.methods.items():
+            for method_name in methods.keys():
+                delattr(self.__class__, method_name)
+
+        self._reset({})
+
+        if self._template is not None:
+            self.AddTemplate(self._template, self._template_parameters)
+
+        objects[self.path] = self
+
     @dbus.service.method(MOCK_IFACE,
                          in_signature='sssss',
                          out_signature='')
@@ -329,16 +410,27 @@ class DBusMockObject(dbus.service.Object):
         except ImportError as e:
             raise dbus.exceptions.DBusException('Cannot add template %s: %s' % (template, str(e)))
 
+        # If the template specifies this is an ObjectManager, set that up
+        if hasattr(module, 'IS_OBJECT_MANAGER') and module.IS_OBJECT_MANAGER:
+            self._set_up_object_manager()
+
         # pick out all D-BUS service methods and add them to our interface
         for symbol in dir(module):
             fn = getattr(module, symbol)
-            if '_dbus_interface' in dir(fn):
-                setattr(self.__class__, symbol, fn)
+            if ('_dbus_interface' in dir(fn) and
+                    ('_dbus_is_signal' not in dir(fn) or not fn._dbus_is_signal)):
+                self.AddMethod(fn._dbus_interface, symbol,
+                               fn._dbus_in_signature, fn._dbus_out_signature,
+                               fn)
 
         if parameters is None:
             parameters = {}
 
         module.load(self, parameters)
+        # save the given template and parameters for re-instantiation on
+        # Reset()
+        self._template = template
+        self._template_parameters = parameters
 
     @dbus.service.method(MOCK_IFACE,
                          in_signature='sssav',
@@ -406,6 +498,12 @@ class DBusMockObject(dbus.service.Object):
 
     @dbus.service.signal(MOCK_IFACE, signature='sav')
     def MethodCalled(self, name, args):
+        '''Signal emitted for every called mock method.
+
+        This is emitted for all mock method calls.  This can be used to confirm
+        that a particular method was called with particular arguments, as an
+        alternative to reading the mock's log or GetCalls().
+        '''
         pass
 
     def mock_method(self, interface, dbus_method, in_signature, *args, **kwargs):
@@ -429,8 +527,13 @@ class DBusMockObject(dbus.service.Object):
         self.call_log.append((int(time.time()), str(dbus_method), args))
         self.MethodCalled(dbus_method, args)
 
+        # The code may be a Python 3 string to interpret, or may be a function
+        # object (if AddMethod was called from within Python itself, rather than
+        # over D-Bus).
         code = self.methods[interface][dbus_method][2]
-        if code:
+        if code and isinstance(code, types.FunctionType):
+            return code(self, *args)
+        elif code:
             loc = locals().copy()
             exec(code, globals(), loc)
             if 'ret' in loc:
